/*
 * qsensview - Environment sensor data view
 *
 * Copyright (C) 2019  B. Stultiens
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __QSENSVIEW_MAINWINDOW_H
#define __QSENSVIEW_MAINWINDOW_H

#include <unistd.h>
#include <math.h>

#include <QWidget>
#include <QPainter>
#include <QImage>
#include <QSerialPort>
#include <QLabel>
#include <QChartView>
#include <QChart>
#include <QByteArray>
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QNetworkReply>

using namespace QtCharts;

class qsvChartView;
class QGridLayout;
class QTabWidget;
class QTimer;

class MeasurementValue
{
public:
	MeasurementValue() {
		reset();
	};

	void reset() {
		n = 0;
		sum = 0.0;
		mini = HUGE_VAL;
		maxi = -HUGE_VAL;
	};

	float getMin()		{ return n > 0 ? mini : 0.0; };
	float getMax()		{ return n > 0 ? maxi : 0.0; };
	float getAvg()		{ return n > 0 ? sum / n : 0.0; };
	bool hasValues()	{ return n != 0; };
	float addValue(float v)	{
		if(v < mini)
			mini = v;
		if(v > maxi)
			maxi = v;
		sum += v;
		n++;
		return v;
	};

private:
	unsigned n;
	float sum;
	float mini;
	float maxi;
};

struct GraphView {
	GraphView() : label(NULL), value(NULL), units(NULL), view(NULL) {};
	QLabel *label;
	QLabel *value;
	QLabel *units;
	qsvChartView *view;
	QString jsontag;
	unsigned dataindex;
	float datafactor;
	float ymin;
	float ymax;
	MeasurementValue measurement;
	int tabindex;
};

#define NGRAPHS	6

class qsvMainWindow : public QWidget
{
Q_OBJECT
public:
	qsvMainWindow();

public slots:
	void nwfinished(QNetworkReply *r);
	void timeout();
	void blTimeout();
	void tabEvent(int);
	void spRead();
	void spError(QSerialPort::SerialPortError);

	void mouseDoubleClickEvent(QMouseEvent *event) override;
	void mouseMoveEvent(QMouseEvent *event) override;
	void mousePressEvent(QMouseEvent *event) override;
	void mouseReleaseEvent(QMouseEvent *event) override;

private:
	GraphView *createGraph(int idx, QGridLayout *g, int col, int row);
	void setBacklight(int val);
	void activity();

private:
	QTabWidget *tw;

	QSerialPort *sp;
	QByteArray spdata;

	bool firstData;			// First data round is junky
	GraphView *graphs[NGRAPHS];
	QList<GraphView> datapoints;

	QTimer *bltimer;
	int blstate;
	int blight;
	int blmin;
	int blmax;
	int blinterval;
	int blwait;

	int datamissing;

	QNetworkAccessManager qnam;
};

#endif
